/**********************************************************************

	Command:  UNDUP  -- Check and remove duplicated entities

	Programmed by Kang, Hsin-Min.
	Released at Oct 6, 1998 Initial release
	Revised at Oct 15, 1998 for Japenese message text translation

	Copyright (c) 1998 TCAM Development House, Taipei
	All rights Reserved.

	Status: TwinCAD Standard Command, with full support from
		TCAM Development House.

  Commants:

	Select Object (+):

***********************************************************************/
//#include "tclcad.h"
// Resolve definition from tclcad.h
#define T_POLY		0
#define T_POINT 	1
#define T_LINE		2
#define T_ARC		3
#define T_CIRCLE	4
#define T_LINE3D	5
#define T_TEXT		6
#define T_BLOCK 	7
#define T_INSERT	8
#define T_DIM		9
#define T_REGION	10
#define T_TAG		11
#define T_SYMBOL	12

#define TGBIG5		1
#define TGGB2312	2
#define TGSJIS		5
#define TGKOREAN	3

char	szText[4][70];		// Our display text buffer
char	szAsciiText[][70]={	// Default English Display Text
/*  0 */   "\nUnDup -- Please select entities for duplication check...",
/*  1 */   "\n## Can not process more than 4000 entities at a time.",
/*  2 */   "\nTotal %d duplicated entities have been removed.",
/*  3 */   "\nNo duplicated entities are found.",
};
char	szBig5Text[][60]={	// Default Chinese Big-5 Display Text
/*  0 */   "\nUnDup -- пni歫ˬdϤ...",
/*  1 */   "\n## Lk@BzWL4000ӹϤˬd.",
/*  2 */   "\n`@ %d ӭƪϤQR.",
/*  3 */   "\nҿϤ, õLo{󭫽ƪϤ.",
};
char	szGB2312Text[][70]={	// Default Chinese GB2312 Display Text
/*  0 */   "\nUnDup -- ѡȡҪظͼԪ...",
/*  1 */   "\n## ޷һδ4000ͼԪظ.",
/*  2 */   "\nܹ %d ظͼԪɾ.",
/*  3 */   "\nѡȡͼԪ, ޷κظͼԪ.",
};
//
//	The following Japenese messages are translated by _____________.
//
char	szSJISText[][70]={    // Default Japenese SJIS Display Text
/*  0 */   "\nUnDup -- d̃`FbN}`̑I",
/*  1 */   "\n## Px 4000 ȏ̐}``FbNł܂B",
/*  2 */   "\nv %d ̏d}`菜܂B",
/*  3 */   "\nd}`͂܂B",
};
//
//	The following Korean messages are translated by _____________.
//
char	szKoreanText[][70]={	// Default Korean Display Text
/*  0 */   "\nUnDup -- Please select entities for duplication check...",
/*  1 */   "\n## Can not process more than 4000 entities at a time.",
/*  2 */   "\nTotal %d duplicated entities have been removed.",
/*  3 */   "\nNo duplicated entities are found.",
};

COMMAND Undup()
{
	//
	// Set up our messages according to current TGF type
	//
	switch(@tgftype) {
	  case TGBIG5:	  setdata(szText,szBig5Text);	  break;
	  case TGGB2312:  setdata(szText,szGB2312Text);   break;
	  case TGSJIS:	  setdata(szText,szSJISText);	  break;
	  case TGKOREAN:  setdata(szText,szKoreanText);   break;
	  default:	  setdata(szText,szAsciiText);	  break;
	}

	//
	// Obtain selection list to check.
	//
	SLIST	sChk;
	int	nChk;

	printf(szText[0]/*"\nUndup -- Please select entities for duplication check..."*/);
	sChk = getesel();
	nChk = ent_count(sChk);
	if ( nChk <= 1 ) return 0;	// Nothing to do
	if ( nChk >= 4000 ) {
		printf(szText[1]/*"\n## Can not process more than 4000 entities at a time."*/);
		return 0;
	}

	POINT	pChk[nChk];
	int	nSortIdx[nChk];
	int	nDupIdx[nChk];
	int	etype;
	int	i, nDupcnt;
	//
	// Take a single characterist point from each entity and store them
	// in an array of POINT. Sort the array and we only need to check
	// adjacent elements for duplication check...
	//
	for(i=0;i<nChk;i++) {
	    switch(ent_type(sChk[i])) {
	      case T_CIRCLE:
	      case T_POINT:	pChk[i] = sChk[i]; continue;
	      case T_LINE:
	      case T_ARC:
	      case T_POLY:	pChk[i] = centroid(sChk[i],1); continue;
	      case T_LINE3D:	pChk[i] = ent_read(sChk[i]).line3d.ps;
	      case T_TAG:	pChk[i] = ent_read(sChk[i]).tag.pbase; continue;
	      case T_TEXT:	pChk[i] = ent_read(sChk[i]).text.pbase; continue;
	      case T_INSERT:	pChk[i] = ent_read(sChk[i]).insert.pbase; continue;
	      case T_REGION:	pChk[i] = ent_read(sChk[i]).region.pbase; continue;
	      case T_DIM:	pChk[i] = ent_read(sChk[i]).dim.ps; continue;
	      case T_SYMBOL:	pChk[i] = ent_read(sChk[i]).symbol.pbase; continue;
	      default:		pChk[i] = p(0,0);
	    }
	}
	sort(pChk,nChk,0,nSortIdx);
	//
	// If the successive two elements are the same, further check for
	// duplication entities are conducted.
	//
	nChk--;
	for(i=nDupcnt=0;i<nChk;i++) {
	    if ( pChk[i]!=pChk[i+1] ) continue;
	    e1 = sChk[nSortIdx[i]];
	    etype = ent_type(e1);
	    if ( etype != ent_type(sChk[nSortIdx[i+1]]) )
		continue;
	    e2 = sChk[nSortIdx[i+1]];
	    switch(etype) {
	      case T_POINT: break;
	      case T_LINE:
		if ( ent_read(e1).line.ps != ent_read(e2).line.ps ) continue;
		break;
	      case T_ARC:
		if ( ent_read(e1).arc.ps != ent_read(e2).arc.ps ) continue;
		break;
	      case T_CIRCLE:
		if ( ent_read(e1).circle.rad != ent_read(e2).circle.rad ) continue;
		break;
	      case T_LINE3D:
		if ( ent_read(e1).line3d.pe != ent_read(e2).line3d.pe ) continue;
		if ( ent_read(e1).line3d.zs != ent_read(e2).line3d.ze ) continue;
		break;
	      case T_POLY:
		if ( centroid(e1) != centroid(e2) ) continue;
		if ( ent_area(e1) != ent_area(e2) ) continue;
		if ( ent_len(e1) != ent_len(e2) ) continue;
		if ( gextent(e1) != gextent(e2) ) continue;
		break;
	      case T_TEXT:
		if ( ent_read(e1).text.str != ent_read(e2).text.str ) continue;
		break;
	      case T_TAG:
		if ( ent_read(e1).tag.str != ent_read(e2).tag.str ) continue;
		if ( ent_read(e1).tag.name!= ent_read(e2).tag.name) continue;
		break;
	      case T_DIM:
		if ( ent_read(e1).dim.pe != ent_read(e2).dim.pe ) continue;
		if ( ent_read(e1).dim.pde != ent_read(e2).dim.pde ) continue;
		if ( ent_read(e1).dim.pds != ent_read(e2).dim.pds ) continue;
		break;
	      case T_SYMBOL:
		if ( ent_read(e1).symbol.name!= ent_read(e2).symbol.name) continue;
		break;
	      case T_INSERT:
		if ( ent_read(e1).insert.name!= ent_read(e2).insert.name) continue;
		if ( ent_read(e1).insert.angle!= ent_read(e2).insert.angle) continue;
		if ( ent_read(e1).insert.pscale!= ent_read(e2).insert.pscale) continue;
		break;
	      case T_INSERT:
		if ( ent_read(e1).region.name!= ent_read(e2).region.name) continue;
		if ( ent_read(e1).region.angle!= ent_read(e2).region.angle) continue;
		if ( ent_read(e1).region.pscale!= ent_read(e2).region.pscale) continue;
		break;
	      default:
		continue;
	    }
	    nDupIdx[nDupcnt++] = nSortIdx[i];
	}
	//
	// Alas, remove the duplicated objects.
	//
	if ( nDupcnt ) {
		SLIST	sDup;
		int	cmdecho;
		sDup = ent_aloc(nDupcnt);
		for(i=0;i<nDupcnt;i++) sDup[i] = sChk[nDupIdx[i]];
		cmdecho = @cmdecho;
		@cmdecho = 0;
		command("ERASE \@sDup ^M");
		@cmdecho = cmdecho;
		printf(szText[2]/*"\nTotal %d duplicated entities have been removed."*/,nDupcnt);
		regen();
	}
	else {
		printf(szText[3]/*"\nNo duplicated entities are found."*/);
	}
}
